/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "spi.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "lcd_hd44780_lib.h"
#include "./FatFs/diskio.h"
#include "./FatFs/ff.h"

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
#define STAN_PROG_OCZEKIWANIE    0
#define STAN_PROG_JEST_KARTA     2
#define STAN_PROG_BRAK_KARTY     3
#define STAN_PROG_BLAD_KARTY     4
#define STAN_PROG_BRAK_PLIKU     5
#define STAN_PROG_ODCZYT_PLIKU   6
/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
char stanProg=STAN_PROG_OCZEKIWANIE;
static FATFS g_sFatFs; //obiekt FATFs

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  unsigned long int i;
  char poprzedniStanGPIOA8=0;
  char obecnyStanGPIOA8=0;

  FRESULT fresult;
  FIL     plik;
  char    bufor[20]={0};
  UINT    odczytanychBajtow=0;
  UINT    bajtowDoOdczytu=0;

  /* USER CODE END 1 */
  

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  /* USER CODE BEGIN 2 */

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */

  LCD_Initialize();                           //inicjalizacja wyswietlacza
  LCD_WriteCommand(HD44780_CLEAR);            //wyczysc wyswietlacz
  LCD_WriteText("SD - DEMO");

  obecnyStanGPIOA8=(1-HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_8));//zanegowanie rzeczywistego stanu GPIOA8 spowoduje, ze stan obecny i stan poprzedni beda rozne -> wymusi to testowanie obecnosci karty przy pierwszym wejsciu w STAN_PROG_OCZEKIWANIE
  HAL_Delay(5);

  while (1)
  {
	poprzedniStanGPIOA8=obecnyStanGPIOA8;
	obecnyStanGPIOA8=HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_8);

	switch (stanProg){
	  case STAN_PROG_OCZEKIWANIE : {
									 if (obecnyStanGPIOA8==poprzedniStanGPIOA8){
									   HAL_Delay(100);
									 } else {
									   if ((obecnyStanGPIOA8==SET)&&(poprzedniStanGPIOA8==RESET)){
										 stanProg=STAN_PROG_BRAK_KARTY;
									   } else {
										 stanProg=STAN_PROG_JEST_KARTA;
									   }
									   HAL_GPIO_WritePin(GPIOB, GPIO_PIN_8, GPIO_PIN_RESET);   //jest inicjalizacja?
									   HAL_GPIO_WritePin(GPIOB, GPIO_PIN_9, GPIO_PIN_RESET);   //jest plik?
									 }
								   } break;
	  case STAN_PROG_BRAK_KARTY : {  //brak karty
									 LCD_WriteTextXY("Brak karty        ",0,1);
									 stanProg=STAN_PROG_OCZEKIWANIE;
								   } break;
	  case STAN_PROG_BLAD_KARTY : {  //blad inicjalizacji (np. blad komunikacji z karta)
									 LCD_WriteTextXY("Blad karty        ",0,1);
									 stanProg=STAN_PROG_OCZEKIWANIE;
								   } break;

	  case STAN_PROG_JEST_KARTA : {
									 HAL_Delay(50);      	                //odczkaj chwile az karta bedzie stabilnie umieszczona w gniezdzie
									 fresult = f_mount(&g_sFatFs,"/",1);
									 if (fresult==0) {
										HAL_GPIO_WritePin(GPIOB, GPIO_PIN_8, GPIO_PIN_SET);   //karta zainicjalizowana
										stanProg=STAN_PROG_ODCZYT_PLIKU;
									 } else {
										stanProg=STAN_PROG_BLAD_KARTY;
									 }
								  } break;
	  case STAN_PROG_BRAK_PLIKU : {
									 LCD_WriteTextXY("Brak pliku        ",0,1);
									 stanProg=STAN_PROG_OCZEKIWANIE;
								   } break;
	  case STAN_PROG_ODCZYT_PLIKU : {
									  fresult = f_open(&plik, "plik.txt", FA_READ);
									  if (fresult==0){                           //jesli uda sie otworzyc plik
										HAL_GPIO_WritePin(GPIOB, GPIO_PIN_9, GPIO_PIN_SET);   //jest plik
										bajtowDoOdczytu=f_size(&plik);
										if (bajtowDoOdczytu>16) {bajtowDoOdczytu=16;} //zabezpieczenie przed przepelnieniem bufora
										fresult = f_read(&plik, bufor, bajtowDoOdczytu, &odczytanychBajtow);
										for (i=odczytanychBajtow; i<16; i++) {
										  bufor[i]=0x20;                  //wypelnienie spacjami od konca tekstu, zeby usunac z wyswietlacza caly poprzedni tekst
										}
										bufor[16]=0;                      //dodanie znaku \0 na koncu tekstu
										LCD_WriteTextXY(bufor,0,1);
										  fresult = f_close (&plik);
										stanProg=STAN_PROG_OCZEKIWANIE;
									  } else {                                   //brak pliku
										stanProg=STAN_PROG_BRAK_PLIKU;
									  }
									 } break;

	  default : stanProg=STAN_PROG_OCZEKIWANIE;
	}
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL9;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */
extern void disk_timerproc(void);

void HAL_SYSTICK_Callback(void){
	disk_timerproc();
}

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
