/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "spi.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "lcd_hd44780_lib.h"
#include "./FATFs/ff.h"
#include "./FATFs/diskio.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
typedef enum {FALSE = 0, TRUE = !FALSE} bool;
/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define STAN_PROG_OCZEKIWANIE     0
#define STAN_PROG_GOTOWOSC_ZAPISU 1
#define STAN_PROG_JEST_KARTA      2
#define STAN_PROG_BRAK_KARTY      3
#define STAN_PROG_BLAD_KARTY      4
#define STAN_PROG_BRAK_PLIKU      5
#define STAN_PROG_ZAPIS_PLIKU     7
#define STAN_PROG_BLAD_ZAPISU     8

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
char stanProg=STAN_PROG_OCZEKIWANIE;
static FATFS g_sFatFs; //obiekt FATFs
bool sekunda=FALSE;

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  unsigned int temperatura;
  char temperaturaTekst[8]={" 0,0 C"};
  const char stopienSymbol[8] = {0x06, 0x09, 0x09, 0x06, 0x00, 0x00, 0x00,0};   //symbol stopnia

  FRESULT fresult;
  FIL plik;
  UINT bajtowZapisanych=0;
  char poprzedniStanGPIOA8=0;
  char obecnyStanGPIOA8=0;
  bool zapis=FALSE;
  unsigned long int t=0;
  char tTekst[8]={"0"};

  /* USER CODE END 1 */
  

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_SPI2_Init();
  /* USER CODE BEGIN 2 */

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */

  LCD_Initialize();                                         //Inicjalizacja wyswietlacza
  LCD_SetUserChar(1, 1, stopienSymbol);                     //Rejestracja symbolu stopnia
  LCD_WriteCommand(HD44780_CLEAR);                          //Wyczysc wyswietlacz
  LCD_WriteText("Temp.:  0,0 C ");

  obecnyStanGPIOA8=(1-HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_8));//zanegowanie rzeczywistego stanu GPIOA8 spowoduje, ze stan obecny i stan poprzedni beda rozne -> wymusi to testowanie obecnosci karty przy pierwszym wejsciu w STAN_PROG_OCZEKIWANIE
  HAL_Delay(5);

  while (1)
  {
	 //Czesc odpowiedzialna za pobranie temperatury z czujnika i jej wyswietlenie na LCD
	 HAL_GPIO_WritePin(GPIOB, GPIO_PIN_12, GPIO_PIN_RESET);   //niski stan nSS
	 while (HAL_SPI_GetState(&hspi2) == !HAL_SPI_STATE_READY); //Czekaj na gotowosc SPI

	 HAL_SPI_Receive(&hspi2, (uint8_t *)&temperatura, 1, 50);//Odbierz dane
	 if ((temperatura&0x04)==0){                             //Sprawdz, czy zakonczono juz pierwszy pomiar po wlaczeniu ukladu TC77
	     temperatura=0;                                      //Jesli nie, ustaw temperature=0, inaczej wynik bedzie bledny (>500stC)
	 }
	 temperatura = temperatura >> 3;                         //Usun 3 LSB, dane sa na bitach 15...3
	 HAL_GPIO_WritePin(GPIOB, GPIO_PIN_12, GPIO_PIN_SET);     //ustaw wysoki stan nSS

	 temperatura = (temperatura * 625)/100;                  //1 bit temperatury odpowiada 0,0625 stopnia
	 sprintf(temperaturaTekst, "%2d,%d C ", temperatura / 100, (temperatura % 100)/10 );
	 temperaturaTekst[4]=1;
	 LCD_WriteTextXY(temperaturaTekst,7,0);
	// HAL_Delay(500);
	 HAL_GPIO_TogglePin(GPIOB,GPIO_PIN_15);

	 //Czesc odpowiedzialna za obsluge karty SD, w tym zapis temperatury do pliku co 1s
	 poprzedniStanGPIOA8=obecnyStanGPIOA8;
	 obecnyStanGPIOA8=HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_8);
	 switch (stanProg){
	     case STAN_PROG_OCZEKIWANIE : {
	                                     if (obecnyStanGPIOA8==poprzedniStanGPIOA8){
	                                       HAL_Delay(100);
	                                     } else {
	                                       if ((obecnyStanGPIOA8==SET)&&(poprzedniStanGPIOA8==RESET)){
	                                         stanProg=STAN_PROG_BRAK_KARTY;
	                                       } else {
	                                         stanProg=STAN_PROG_JEST_KARTA;
	                                       }
	                                     }
	                                   } break;
	      case STAN_PROG_GOTOWOSC_ZAPISU : {
	                                     if (obecnyStanGPIOA8==poprzedniStanGPIOA8){ //brak zmiany stanu oznacza, ze nie wyjeto karty w czasie zapisu
		                                   HAL_Delay(100);

	                                       if (!HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_0)){//SW0 - wlacz zapis
	                                         zapis=TRUE;
	                                         t=0;
	                                       }
	                                       if (!HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_1)){//SW1 - wylacz zapis
	                                         zapis=FALSE;
	                                       }

	                                       if (zapis) {                              //trwa zapis
	                                         LCD_WriteTextXY("Zapis pliku       ",0,1);
	                                         HAL_GPIO_WritePin(GPIOB, GPIO_PIN_11, GPIO_PIN_SET);
	                                         if (sekunda) {                          //zapis kolejnych wartosci tylko w chwilach, gdy uplynela kolejna sekunda
	                                           stanProg=STAN_PROG_ZAPIS_PLIKU;
	                                         }
	                                       } else {                                  //brak zapisu, ale karta zainicjalizowana i gotowa do zapisu
	                                         LCD_WriteTextXY("Gotowy do zapisu  ",0,1);
	                                         HAL_GPIO_WritePin(GPIOB, GPIO_PIN_11, GPIO_PIN_RESET);
	                                       }
	                                     } else {                                    //nastapila zmiana stanu linii DETECT
	                                       if ((obecnyStanGPIOA8==SET)&&(poprzedniStanGPIOA8==RESET)){ //wyjeto karte
	                                         stanProg=STAN_PROG_BRAK_KARTY;
	                                       }
	                                     }
	                                   } break;
	      case STAN_PROG_BRAK_KARTY : {
	                                     LCD_WriteTextXY("Brak karty        ",0,1);
	                                     stanProg=STAN_PROG_OCZEKIWANIE;
	                                     zapis=FALSE;                                //jesli brak karty - zapis jest anulowany
	                                     HAL_GPIO_WritePin(GPIOB, GPIO_PIN_11, GPIO_PIN_RESET);
	                                   } break;
	      case STAN_PROG_BLAD_KARTY : {  //blad inicjalizacji (np. blad komunikacji z karta)
	                                     LCD_WriteTextXY("Blad karty        ",0,1);
	                                     stanProg=STAN_PROG_OCZEKIWANIE;
	                                   } break;
	      case STAN_PROG_JEST_KARTA : {
	                                     fresult = f_mount(&g_sFatFs,"/",1);
	                                     if (fresult==0) {
	                                        stanProg=STAN_PROG_GOTOWOSC_ZAPISU;
	                                     } else {
	                                        stanProg=STAN_PROG_BLAD_KARTY;
	                                     }
	                                  } break;
	      case STAN_PROG_BLAD_ZAPISU : {
	                                     LCD_WriteTextXY("Blad zapisu   ",0,1);
	                                     zapis=FALSE;                                //jesli byl blad - zapis jest anulowany
	                                     HAL_GPIO_WritePin(GPIOB, GPIO_PIN_11, GPIO_PIN_RESET);   //brak zapisu
	                                     stanProg=STAN_PROG_OCZEKIWANIE;
	                                   } break;
	      case STAN_PROG_ZAPIS_PLIKU : {
	    	                             HAL_GPIO_WritePin(GPIOB, GPIO_PIN_10, GPIO_PIN_SET);     //rozpoczeto zapis
	                                     fresult = f_open (&plik, "dane.txt", FA_WRITE | FA_OPEN_ALWAYS);
	                                     if (fresult) {stanProg=STAN_PROG_BLAD_ZAPISU; break;}
	                                     if (t>0) {                                  //jesli czas >0, dopisujemy na koncu pliku nowa linie danych
	                                       fresult = f_lseek(&plik, plik.obj.objsize);
	                                     } else {                                    //jesli czas ==0, to oznacza to, ze zapis dopiero sie zaczyna i nalezy plik "obciac" i zaczac zapis od nowa
	                                       fresult = f_lseek(&plik, 0);
	                                       fresult = f_truncate(&plik);
	                                     }
	    		                         sprintf(tTekst, "%5ld  ", t);                                                    //kolumna czasu
	                                     fresult = f_write (&plik, (const void*)tTekst, 6, &bajtowZapisanych);
	                                     if (fresult) {stanProg=STAN_PROG_BLAD_ZAPISU; break;}
	                                     fresult = f_write (&plik, (const void*)temperaturaTekst, 4, &bajtowZapisanych);	 //wartosc temperatury, zapis 4 bajtow czyli tylko wartosci, bez symbolu stC
	                                     if (fresult) {stanProg=STAN_PROG_BLAD_ZAPISU; break;}
	                                     fresult = f_write (&plik, (const void*)"\r\n", 2, &bajtowZapisanych);	           //nowa linia  (wg. standardu Windows)
	                                     if (fresult) {stanProg=STAN_PROG_BLAD_ZAPISU; break;}
	                                   	 fresult = f_close (&plik);
	                                   	 HAL_GPIO_WritePin(GPIOB, GPIO_PIN_10, GPIO_PIN_RESET);   //koniec zapisu
	                                     stanProg=STAN_PROG_GOTOWOSC_ZAPISU;
	                                     t+=1;                                       //czas dla nastepnej wartosci
	                                   } break;

	      default : stanProg=STAN_PROG_OCZEKIWANIE;
	 }
	 sekunda=FALSE;


    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL9;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */
extern void disk_timerproc (void);

void HAL_SYSTICK_Callback(void){
  static int licznik=0;

  disk_timerproc();
  licznik++;
  if (licznik==1000) {
    sekunda=TRUE;
    licznik=0;
  }
}

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
