/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "adc.h"
#include "dma.h"
#include "spi.h"
#include "tim.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "st7735s_spi_lib.h"
#include "font1.h"
//#define ARM_MATH_CM3   dodane w ustawieniach projektu
#include "arm_math.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
#define N 256                      //liczba prazkow widma
long int buforADC[2*N]={0};        //bufor zmierzonych probek sygnalu (2 polowki, w danej chwili FFT liczone tylko dla polowy danych)
long int buforFFT[N]={0};          //wyznaczone zespolone wartosci widma
long int buforFFTAmp[N]={0};       //wyznaczone prazki widma amplitudowego
volatile unsigned char Transfer=0; //sygnalizacja, ktora polowa buforaADC ma byc przetwarzana 0-dane nie gotowe, 1-pierwsza, 2-druga polowa
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  volatile unsigned long int i;
  char Tekst[5] = {"0"};
  arm_rfft_instance_q31 RFFTInitStruct;  //struktura pomocnicza dla funkcji RFFT
  char tryb=0;                           //tryb pracy 1-widmo w postaci tekstowej, 2-widmo w postaci graficznej
  /* USER CODE END 1 */
  

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_DMA_Init();
  MX_ADC1_Init();
  MX_TIM3_Init();
  /* USER CODE BEGIN 2 */

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */

  //inicjalizacja wyswietacza
  ST7735_SPI_HwConfig();
  ST7735_SPI_Init();

  //kalibracja ADC
  HAL_ADC_Stop(&hadc1);
  HAL_ADCEx_Calibration_Start(&hadc1);

  //Uruchomienie licznika i generowanego przez niego triggera
  HAL_TIM_Base_Start(&htim3);

  //Uruchomienie DMA i wyzwolenie pomiaru
  HAL_ADC_Start_DMA(&hadc1, (uint32_t*)buforADC,2*N);

  //inicjalizacja struktury RFFTInitStruct
  arm_rfft_init_q31 (&RFFTInitStruct, N, 0, 1);

  while (1)
  {
    if (Transfer>0) {
      //obliczenie FFT dla danych z jednej z polowek bufora
      arm_rfft_q31(&RFFTInitStruct, buforADC+(Transfer-1)*N, buforFFT);

      buforFFTAmp[0]=buforFFT[0];       //skladowa stala bez zmian
      for (i = 1; i <= N/2-1; i++) {    //pozostale prazki *2 i przeliczenie tylko polowy probek (reszta symetryczna)
   	  buforFFTAmp[i] = (q31_t)(sqrt((buforFFT[i*2] * buforFFT[i*2]) + (buforFFT[i*2+1] * buforFFT[i*2+1])))*2;
      }
      Transfer=0;

      if (!HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_0)) {      //Widmo w formie wartosci liczbowych - pierwsze 80 prazkow
        if (tryb!=1){
          ST7735_ClearScreen(RGB(0,0,0));
        }

        for (i=0; i<80; i++){
          sprintf(Tekst, "%4ld", buforFFTAmp[i]);              //wyswietlenie probek FFT
          //sprintf(Tekst, "%4ld", buforADC[(Transfer-1)*N]+i);//kontrolne wyswietlenie surowych probek z bufora ADC w celu sprawdzenia, czy dziala czesc pomiarowa programu (TIM-DMA-ADC)
          ST7735_WriteTextXY(Tekst, 2+(i/20)*33, 0+(i-20*(i/20))*8, RGB(255,255,255), RGB(30,70,30), font1 );
        }
        tryb=1;

      } else {                                        //Widmo w formie graficznej - 128 prazkow
       if (tryb!=2){                                  //Czyszczenie ekranu i rysowanie elementow stalych tylko raz, przy zmianie trybu
    	 ST7735_ClearScreen(RGB(0,0,0));
    	 ST7735_WriteTextXY("Widmo amplitudowe FFT\0", 1, 3, RGB(0,120,200), RGB(0,0,0), font1 );
    	 ST7735_FillRect(  0, 15, 127, 159, RGB(170,170,170));
    	 ST7735_FillRect(  0, 30, 127, 95,  RGB(0,0,0));
    	 ST7735_WriteTextXY("0\0",     1, 100, RGB(0,0,0), RGB(170,170,170), font1 ); //opisy osi czestotliwosci
    	 ST7735_FillRect(  0, 95,   0,  98, RGB(0,0,255));
    	 ST7735_WriteTextXY("125\0",  23, 100, RGB(0,0,0), RGB(170,170,170), font1 );
    	 ST7735_FillRect( 31, 95,  31, 98, RGB(0,0,255));
    	 ST7735_WriteTextXY("250\0",  55, 100, RGB(0,0,0), RGB(170,170,170), font1 );
    	 ST7735_FillRect( 63, 95,  63, 98, RGB(0,0,255));
    	 ST7735_WriteTextXY("375\0",  87, 100, RGB(0,0,0), RGB(170,170,170), font1 );
    	 ST7735_FillRect( 95, 95,  95, 98, RGB(0,0,255));
    	 ST7735_WriteTextXY("500\0", 110, 100, RGB(0,0,0), RGB(170,170,170), font1 );
    	 ST7735_FillRect(127, 95, 127, 98, RGB(0,0,255));

    	 ST7735_WriteTextXY("[Hz]\0", 52, 110, RGB(0,0,0), RGB(170,170,170), font1 );
    	 ST7735_WriteTextXY("fs=1kHz\0", 44, 122, RGB(0,0,0), RGB(170,170,170), font1 );
    	 ST7735_WriteTextXY("FFT 256 punktowe\0", 15, 134, RGB(0,0,0), RGB(170,170,170), font1 );
    	 ST7735_WriteTextXY("128 pr\x89""\x90""k\x8E""w widma\0", 11, 144, RGB(0,0,0), RGB(170,170,170), font1 );
         tryb=2;
       }
       for (i=0; i<N/2; i++){                         //rysowanie poszczegolnych prazkow widma i kasowanie poprzednich
    	   ST7735_FillRect(i, 95-buforFFTAmp[i]/64, i, 95, RGB(0,255,0));
    	   ST7735_FillRect(i,                   30, i, 95-buforFFTAmp[i]/64, RGB(0,0,0));
       }
     }

      while (!HAL_GPIO_ReadPin(GPIOA,GPIO_PIN_3)); //SW3 - wstrzymanie aktualizacji wyswietlacza w celu obejrzenia probek widma
    }

    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL9;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_ADC;
  PeriphClkInit.AdcClockSelection = RCC_ADCPCLK2_DIV6;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */
void HAL_ADC_ConvHalfCpltCallback(ADC_HandleTypeDef* hadc){

	//Zakonczono transfer pierwszej polowy probek
	Transfer =1;
	HAL_GPIO_TogglePin(GPIOB,GPIO_PIN_8);
}


void HAL_ADC_ConvCpltCallback(ADC_HandleTypeDef* hadc){

	//Zakonczono transfer drugiej polowy probek
	Transfer =2;
	HAL_GPIO_TogglePin(GPIOB,GPIO_PIN_9);
}
/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
